/*
 -------------------------------------------------------------------------------
 | Producer - Copyright (C) 2001-2007  Don Burns                               |
 |                                                                             |
 | This library is free software; you can redistribute it and/or modify        |
 | it under the terms of the GNU Lesser General Public License as published    |
 | by the Free Software Foundation; either version 3 of the License, or        |
 | (at your option) any later version.                                         |
 |                                                                             |
 | This library is distributed in the hope that it will be useful, but         |
 | WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY  |
 | or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public     |
 | License for more details.                                                   |
 |                                                                             |
 | You should have received a copy of the GNU Lesser General Public License    |
 | along with this software; if not, write to the Free Software Foundation,    |
 | Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.               |
 |                                                                             |
 -------------------------------------------------------------------------------
 */

#include <stdio.h>
#include <stdlib.h>

#include <osgDB/ReadFile>
#include <osg/Node>
#include <osgUtil/SceneView>

#include <Producer/Camera.h>
#include <ProducerOSG/OsgSceneHandler.h>

#include <Producer/Trackball.h>

#include <FL/Fl.H>
#include <FL/Fl_Window.H>
#include <FL/Fl_Sys_Menu_Bar.H>
#include <FL/Fl_Roller.H>

#include "FLTKRenderSurface.h"

Producer::ref_ptr<Producer::Trackball> trackball;

static void somecallback(Fl_Widget* w, void* v) 
{
}

static void quit_cb(Fl_Widget*, void*) 
{
    exit(0);
}

/* To create menu shortcuts that are the same on all platforms (Windows, *nix, 
   Mac) then it is recommended to use FL_COMMAND with shortcuts. This 
   translates to FL_CTRL on Windows and *nix, and the apple key on Mac. Also,
   to promote better Mac look and feel with a single system menu bar, it is 
   recommended to use Fl_Sys_Menu_Bar instead of Fl_Menu_Bar, which does the
   right thing on all platforms. */
static Fl_Menu_Item menuitems[] = {
  { "&File",              0, 0, 0, FL_SUBMENU },
    { "&New",        FL_COMMAND | 'n', (Fl_Callback *)somecallback },
    { "&Open...",    FL_COMMAND | 'o', (Fl_Callback *)somecallback },
    { "&Save",       FL_COMMAND | 's', (Fl_Callback *)somecallback },
    { "Save &As...", FL_COMMAND | FL_SHIFT | 'a', (Fl_Callback *)somecallback, 0, FL_MENU_DIVIDER },
    { "&Quit",       FL_COMMAND | 'q', (Fl_Callback *)quit_cb, 0 },
    { 0 },

  { "&Edit", 0, 0, 0, FL_SUBMENU },
    { "Cu&t",        FL_COMMAND | 'x', (Fl_Callback *)somecallback },
    { "&Copy",       FL_COMMAND | 'c', (Fl_Callback *)somecallback },
    { "&Paste",      FL_COMMAND | 'v', (Fl_Callback *)somecallback },
    { "Select &All", FL_COMMAND | 'a', (Fl_Callback *)somecallback },
    { 0 },


  { "&Help", 0, 0, 0, FL_SUBMENU },
    { "&About...",       FL_COMMAND | 'f', (Fl_Callback *)somecallback, 0, FL_MENU_DIVIDER },
    { "&Reference...",   FL_COMMAND | 'g', (Fl_Callback *)somecallback },
    { "&Tutorial...",    FL_COMMAND | 'h', (Fl_Callback *)somecallback },
    { 0 },

  { 0 }
};

class ProducerWidget: public FLTKRenderSurface
{
    public:
        ProducerWidget( Producer::Camera::SceneHandler *sh, 
                int X, int Y, int W, int H, const char *L=""):
                FLTKRenderSurface( X, Y, W, H, L )
        {
            _camera = new Producer::Camera;
            _camera->setSceneHandler(sh);
            _camera->setRenderSurface( this );

            Producer::Vec3 sceneCenter;
            double sceneRadius;
            sh->getSceneBounds( sceneCenter, sceneRadius );

            _trackball = new Producer::Trackball;
            _trackball->setOrientation( Producer::Trackball::Z_UP );
            _trackball->translate(-sceneCenter[0],-sceneCenter[1],-sceneCenter[2]);
            _trackball->setDistance(sceneRadius*3.0f);
            _trackball->setReference();

            setDrawCallback( s_draw_cb, this );
        }

        static void s_draw_cb( Fl_Widget *, void *data )
        {
            ProducerWidget *pw = static_cast<ProducerWidget *>(data);
            if( pw != NULL )
                pw->redraw();
        }

        void redraw()
        {
            _camera->setViewByMatrix( _trackball->getMatrix() );
            _camera->frame();

            float nx, ny;
            _normalizeMouseXY( _mx, _my, nx, ny );
            _trackball->input( nx, ny, _mbutton );
                _camera->frame();
        }

    private:
        Producer::ref_ptr<Producer::Camera> _camera;
        Producer::ref_ptr<Producer::Trackball> _trackball;


        void _normalizeMouseXY( int x, int y, float &nx, float &ny )
        {
            int wx, wy;
            unsigned int ww, wh;
            getWindowRectangle( wx, wy, ww, wh );

            nx = -1.0 + (2 * (float)x/(float)ww);
            ny =  1.0 - (2 * (float)y/(float)wh);
        }
};

int main(int argc, char **argv)
{
    // Load the database
    osg::ref_ptr<osg::Node> node =  osgDB::readNodeFile( "cow.osg" );
    if(  !node.valid() )
        return 1;

    /////////////////////////////////////////////////////////////////////
    // Initialize FLTK stuff
    Fl_Window *window = new Fl_Window( 660, 560, "FLTK Example" );
    Fl_Sys_Menu_Bar* m = new Fl_Sys_Menu_Bar(0, 0, 660, 30);
    m->menu(menuitems);

    // Set up the scene handler
    Producer::ref_ptr<Producer::OsgSceneHandler> osgSceneHandler = new Producer::OsgSceneHandler;
    if( node.valid() )
        osgSceneHandler->setScene( node.get() );

    // Create the Producer Widget, containing the Trackball and Camera
    Producer::ref_ptr<ProducerWidget> pw =  new ProducerWidget(osgSceneHandler.get(), 10, 40, 640, 480 );

    window->end();
    window->show(argc, argv);

    return Fl::run();
}

